using System.Collections.ObjectModel;
using Phares.Shared.Models;
using Phares.Shared.Models.Properties;
using Phares.Shared.Models.Stateless;

namespace View_by_Distance.Distance.Models.Stateless;

internal static class FilterLogicB
{

    internal static ReadOnlyCollection<LocationContainer> GetLocationContainers(DistanceSettings distanceSettings, ICompareSettings compareSettings, ICompare compare, ReadOnlyCollection<ExifDirectory> exifDirectories, string sourceClass)
    {
        List<LocationContainer> results = [];
        int maxDegreeOfParallelism = compareSettings.MaxDegreeOfParallelism;
        ParallelOptions parallelOptions = new() { MaxDegreeOfParallelism = maxDegreeOfParallelism };
        int totalSeconds = (int)Math.Floor(new TimeSpan(DateTime.Now.Ticks - compare.Ticks).TotalSeconds);
        string message = $") Building LocationContainers Face Files Collection {sourceClass} - {totalSeconds} total second(s)";
        compare.ConstructProgressBar(exifDirectories.Count, message);
        _ = Parallel.For(0, exifDirectories.Count, parallelOptions, (i, state) =>
            LocationContainersParallelFor(distanceSettings, compareSettings, compare, exifDirectories, i, results));
        return results.AsReadOnly();
    }

    private static void LocationContainersParallelFor(DistanceSettings distanceSettings, ICompareSettings compareSettings, ICompare compare, ReadOnlyCollection<ExifDirectory> exifDirectories, int i, List<LocationContainer> results)
    {
        compare.Tick();
        ExifDirectory exifDirectory = exifDirectories[i];
        if (exifDirectory.FilePath.Id is null)
            return;
        LocationContainer? locationContainer = FaceEncodingLogic.GetLocationContainer(distanceSettings, compareSettings, exifDirectory);
        if (locationContainer is null)
            return;
        lock (results)
            results.Add(locationContainer);
    }

}