using System.Collections.ObjectModel;
using Phares.Metadata.Models;
using Phares.Shared.Models;
using Phares.Shared.Models.Stateless;

namespace View_by_Distance.Distance.Models.Stateless;

internal static class FilterLogicC
{

    internal static ReadOnlyCollection<LocationContainer> GetPostFilterLocationContainer(ReadOnlyCollection<LocationContainer> preFiltered, DistanceLimits distanceLimits)
    {
        List<LocationContainer> results = [];
        foreach (LocationContainer locationContainer in preFiltered)
        {
            if (locationContainer.FaceFile is null)
                continue;
            if (locationContainer.FaceFile.AreaPermyriad < distanceLimits.FaceAreaPermyriad)
                continue;
            if (locationContainer.FaceFile.ConfidencePercent < distanceLimits.FaceConfidencePercent)
                continue;
            results.Add(locationContainer);
        }
        return results.AsReadOnly();
    }

    private static ReadOnlyCollection<LocationContainer> GetCombined(ReadOnlyDictionary<int, ReadOnlyDictionary<int, LocationContainer>> mappedIdsThenWholePercentagesToLocationContainer, ReadOnlyCollection<LocationContainer> postFiltered)
    {
        List<LocationContainer> results = [];
        foreach (LocationContainer locationContainer in postFiltered)
            results.Add(locationContainer);
        foreach (KeyValuePair<int, ReadOnlyDictionary<int, LocationContainer>> keyValuePair in mappedIdsThenWholePercentagesToLocationContainer)
        {
            foreach (KeyValuePair<int, LocationContainer> keyValue in keyValuePair.Value)
                results.Add(keyValue.Value);
        }
        return results.AsReadOnly();
    }

    internal static ReadOnlyCollection<LocationContainer> GetMatrixLocationContainers(DistanceSettings distanceSettings, ICompare compare, ReadOnlyDictionary<int, ReadOnlyDictionary<int, LocationContainer>> mappedIdsThenWholePercentagesToLocationContainer, DistanceLimits distanceLimits, ReadOnlyCollection<LocationContainer> postFiltered)
    {
        List<LocationContainer> results = [];
        ReadOnlyCollection<LocationContainer> collection;
        ReadOnlyCollection<LocationContainer> locationContainers = GetCombined(mappedIdsThenWholePercentagesToLocationContainer, postFiltered);
        string message = $") Building Matrix - {(int)Math.Floor(new TimeSpan(DateTime.Now.Ticks - compare.Ticks).TotalSeconds)} total second(s)";
        compare.ConstructProgressBar(postFiltered.Count, message);
        foreach (LocationContainer locationContainer in postFiltered)
        {
            compare.Tick();
            if (locationContainer.FilePath.Id is null)
                continue;
            collection = FaceRecognitionDotNet.Models.FaceRecognition.GetLocationContainers(distanceSettings.FaceDistancePermyriad, locationContainers, locationContainer);
            foreach (LocationContainer l in collection)
            {
                // if (locationContainer.FilePath.Id == -626078035 && l.FilePath.Id == -626078035)
                //     continue;
                if (l.FilePath.Id is null)
                    continue;
                if (l.LengthPermyriad is null)
                    continue;
                if (l.LengthPermyriad > distanceLimits.FaceDistancePermyriad)
                    break;
                if (!distanceSettings.SaveSortingWithoutPerson && l.PersonKeyFormattedAndKeyTicksAndDisplayDirectoryName is null)
                    continue;
                if (l.FilePath.Id.Value == locationContainer.FilePath.Id.Value && l.WholePercentages == locationContainer.WholePercentages)
                    continue;
                results.Add(l);
            }
        }
        LocationContainer[] array = results.OrderBy(l => l.LengthPermyriad).ToArray();
        return array.AsReadOnly();
    }

    internal static ReadOnlyDictionary<string, LocationContainer> GetOnlyOne(DistanceSettings distanceSettings, ReadOnlyCollection<LocationContainer> matrix)
    {
        Dictionary<string, LocationContainer> results = [];
        List<string> added = [];
        LocationContainer? tryGetValue;
        foreach (LocationContainer locationContainer in matrix)
        {
            if (distanceSettings.SaveIndividually)
                break;
            if (locationContainer.LengthSource is null)
                continue;
            if (distanceSettings.UseExtraPersonKeyCheck)
            {
                if (results.TryGetValue(locationContainer.LengthSource.Name, out tryGetValue))
                {
                    if (locationContainer.PersonKeyFormattedAndKeyTicksAndDisplayDirectoryName is not null && tryGetValue.PersonKeyFormattedAndKeyTicksAndDisplayDirectoryName is not null && locationContainer.PersonKeyFormattedAndKeyTicksAndDisplayDirectoryName.KeyTicks != tryGetValue.PersonKeyFormattedAndKeyTicksAndDisplayDirectoryName.KeyTicks)
                        _ = results.Remove(locationContainer.LengthSource.Name);
                    continue;
                }
            }
            if (added.Contains(locationContainer.LengthSource.Name))
                continue;
            added.Add(locationContainer.LengthSource.Name);
            results.Add(locationContainer.LengthSource.Name, locationContainer);
        }
        return results.AsReadOnly();
    }

}